import { Injectable, UnauthorizedException } from '@nestjs/common';
import { TicketDataService } from '../data/services/ticket.service';
import { QueueOrder } from './entities/order.entity';
import { QueueItemModel } from '../data/models/queue.model';

@Injectable()
export class QueueOrchestrator {
  constructor(private readonly dataService: TicketDataService) {}

  async loginCustomer(id: string): Promise<QueueOrder> {
    try {
      return await this.dataService.loginQueue(id);
    } catch (error) {
      throw new UnauthorizedException({
        message: 'Invoice tidak ditemukan',
        error: 'INVOICE_NOT_FOUND',
      });
    }
  }

  async queueTickets(order_id: string): Promise<any> {
    const tickets = await this.dataService.queueTickets(order_id);
    return tickets.map((ticket) => {
      const totalActivities = ticket.items.length;
      const totalTickets = ticket.items.reduce(
        (acc, item) => acc + item.qty,
        0,
      );
      const totalUsed = 0;
      const totalQueue = 0;
      return {
        id: ticket.id,
        code: ticket.code,
        customer: ticket.customer,
        phone: ticket.phone,
        date: ticket.date,
        summary: {
          total_activities: totalActivities,
          total_tickets: totalTickets,
          total_used: totalUsed,
          total_queue: totalQueue,
        },
      };
    });
  }

  async queueTicketDetail(order_id: string, ticket_id: string): Promise<any> {
    const tickets = await this.dataService.queueTicketItems(
      order_id,
      ticket_id,
    );
    return tickets.map((ticket) => {
      const totalActivities = ticket.items.length;
      const totalTickets = ticket.items.reduce(
        (acc, item) => acc + item.qty,
        0,
      );
      const totalUsed = 0;
      const totalQueue = 0;
      return {
        id: ticket.id,
        code: ticket.code,
        customer: ticket.customer,
        phone: ticket.phone,
        date: ticket.date,
        summary: {
          total_activities: totalActivities,
          total_tickets: totalTickets,
          total_used: totalUsed,
          total_queue: totalQueue,
        },
        items: ticket.items.map((item) => {
          return {
            id: item.item_id,
            title: item.item.item_queue?.name ?? item.item.name,
            image_url: item.item.image_url,
            summary: {
              total_tickets: item.qty,
              total_used: 0,
              total_queue: 1,
            },
            queue: [
              {
                code: 'A001',
                qty: 1,
                time: '15:00',
                status: 'waiting',
              },
            ],
            queue_condition: {
              available: true,
              average: 12,
              nearest: '13:10',
              crowded_level: 20,
              available_time: '15:00',
            },
          };
        }),
      };
    });
  }

  async queueItemDetail(order_id: string, item_id: string): Promise<any> {
    const tickets = await this.dataService.queueItemTickets(order_id, item_id);
    return tickets.map((ticket) => {
      const totalActivities = ticket.items.length;
      const totalTickets = ticket.items.reduce(
        (acc, item) => acc + item.qty,
        0,
      );
      const totalUsed = 0;
      const totalQueue = 0;
      return {
        id: ticket.id,
        code: ticket.code,
        customer: ticket.customer,
        phone: ticket.phone,
        date: ticket.date,
        summary: {
          total_activities: totalActivities,
          total_tickets: totalTickets,
          total_used: totalUsed,
          total_queue: totalQueue,
        },
        queue: [
          {
            code: 'A001',
            qty: 1,
            time: '15:00',
            status: 'waiting',
          },
        ],
      };
    });
  }

  async queueItems(order_id: string): Promise<any> {
    const tickets = await this.dataService.queueTickets(order_id);
    const ticketItems = {};

    tickets.forEach((ticket) => {
      ticket.items.forEach((item) => {
        const item_id = item.item.item_queue?.id ?? item.item.id;
        const currentItem = ticketItems[item_id];
        ticketItems[item_id] = currentItem ? [...currentItem, item] : [item];
      });
    });

    return Object.values<QueueItemModel[]>(ticketItems).map((items) => {
      const item = items[0];
      const item_qty = items.reduce((acc, item) => acc + item.qty, 0);
      return {
        id: item.item_id,
        title: item.item.item_queue?.name ?? item.item.name,
        image_url: item.item.image_url,
        qty: item_qty,

        available: true,
        average: 12,
        nearest: '13:10',
        crowded_level: 20,
        available_time: '15:00',
      };
    });
  }
}
